package ocean.gateway.filters;

import java.lang.reflect.Field;
import java.time.Duration;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.util.Assert;
import org.springframework.web.reactive.function.client.ClientResponse;
import org.springframework.web.reactive.function.client.WebClient;

import ocean.gateway.service.ServiceApplication;
import ocean.gateway.service.filters.TokenFilter;


@RunWith(SpringRunner.class)
@SpringBootTest(classes = {ServiceApplication.class}, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)

public class TokenFilterTests {
    
    @LocalServerPort
    int randomServerPort;

    // private WebTestClient requests;

    String baseUri;
    WebClient request;
    
	@Before
	public void before() {

		request = WebClient.create();
		baseUri = "http://127.0.0.1:" + randomServerPort;
	}


    @Test
    public void GetHelloWithoutToken() {
        
        ClientResponse response = request.get().uri(baseUri + "/hello").exchange().block();


        Assert.notNull(response, "The class must not be null");
        Assert.state(response.statusCode() == HttpStatus.UNAUTHORIZED, "response.getStatusCode() must be HttpStatus.UNAUTHORIZED");
    }

    @Test
    public void GetHelloWithToken() {
        ClientResponse response = request.get().uri(baseUri + "/hello").header("token", TokenFilter.TEST_TOKEN).exchange().block();

        Assert.notNull(response, "The class must not be null");
        Assert.state(response.statusCode() != HttpStatus.UNAUTHORIZED, "response.getStatusCode() must not be HttpStatus.UNAUTHORIZED");
    }
}